@@ -7,6 +7,7 @@
  * Daniel Veillard <daniel@veillard.com>
  */
 
+#define IN_LIBXML
 #include "libxml.h"
 #ifdef LIBXML_DEBUG_ENABLED
 
@@ -695,11 +696,7 @@ xmlDebugDumpOneNode(FILE * output, xmlNodePtr node, int depth)
             shift[2 * i + 2] = shift[2 * i + 3] = 0;
             fprintf(output, shift);
             fprintf(output, "content=");
-#ifndef XML_USE_BUFFER_CONTENT
             xmlDebugDumpString(output, node->content);
-#else
-            xmlDebugDumpString(output, xmlBufferContent(node->content));
-#endif
             fprintf(output, "\n");
         }
     } else {
@@ -1105,11 +1102,7 @@ xmlLsCountNode(xmlNodePtr node) {
 	case XML_PI_NODE:
 	case XML_COMMENT_NODE:
 	    if (node->content != NULL) {
-#ifndef XML_USE_BUFFER_CONTENT	    
 		ret = xmlStrlen(node->content);
-#else
-		ret = xmlBufferLength(node->content);
-#endif
             }
 	    break;
 	case XML_ENTITY_REF_NODE:
@@ -1156,7 +1149,7 @@ xmlLsOneNode(FILE *output, xmlNodePtr node) {
 	    fprintf(output, "t");
 	    break;
 	case XML_CDATA_SECTION_NODE:
-	    fprintf(output, "c");
+	    fprintf(output, "C");
 	    break;
 	case XML_ENTITY_REF_NODE:
 	    fprintf(output, "e");
@@ -1215,11 +1208,7 @@ xmlLsOneNode(FILE *output, xmlNodePtr node) {
 	    break;
 	case XML_TEXT_NODE:
 	    if (node->content != NULL) {
-#ifndef XML_USE_BUFFER_CONTENT	    
 		xmlDebugDumpString(output, node->content);
-#else
-		xmlDebugDumpString(output, xmlBufferContent(node->content));
-#endif
             }
 	    break;
 	case XML_CDATA_SECTION_NODE:
@@ -1357,38 +1346,59 @@ xmlShellPrintXPathError(int errorType, const char *arg)
 
 
 /**
- * xmlShellPrintNode:
- * @node : a non-null node to print to stdout
+ * xmlShellPrintNodeCtxt:
+ * @ctxt : a non-null shell context
+ * @node : a non-null node to print to the output FILE
  *
- * Print node to stdout
+ * Print node to the output FILE
+ */
+static void
+xmlShellPrintNodeCtxt(xmlShellCtxtPtr ctxt,xmlNodePtr node)
+{
+    FILE *fp;
+
+    if (!node)
+        return;
+    if (ctxt == NULL)
+	fp = stdout;
+    else
+	fp = ctxt->output;
+
+    if (node->type == XML_DOCUMENT_NODE)
+        xmlDocDump(fp, (xmlDocPtr) node);
+    else if (node->type == XML_ATTRIBUTE_NODE)
+        xmlDebugDumpAttrList(fp, (xmlAttrPtr) node, 0);
+    else
+        xmlElemDump(fp, node->doc, node);
+
+    fprintf(fp, "\n");
+}
+
+/**
+ * xmlShellPrintNode:
+ * @node : a non-null node to print to the output FILE
+ *
+ * Print node to the output FILE
  */
 void
 xmlShellPrintNode(xmlNodePtr node)
 {
-    if (!node)
-        return;
-
-    if (node->type == XML_DOCUMENT_NODE)
-        xmlDocDump(stdout, (xmlDocPtr) node);
-    else if (node->type == XML_ATTRIBUTE_NODE)
-        xmlDebugDumpAttrList(stdout, (xmlAttrPtr) node, 0);
-    else
-        xmlElemDump(stdout, node->doc, node);
-
-    fprintf(stdout, "\n");
+    xmlShellPrintNodeCtxt(NULL, node);
 }
 
-
 /**
- * xmlShellPrintXPathResult:
+ * xmlShellPrintXPathResultCtxt:
+ * @ctxt: a valid shell context
  * @list: a valid result generated by an xpath evaluation
  *
- * Prints result to stdout
+ * Prints result to the output FILE
  */
-void
-xmlShellPrintXPathResult(xmlXPathObjectPtr list)
+static void
+xmlShellPrintXPathResultCtxt(xmlShellCtxtPtr ctxt,xmlXPathObjectPtr list)
 {
     int i = 0;
+    if (!ctxt)
+       return;
 
     if (list != NULL) {
         switch (list->type) {
@@ -1400,8 +1410,8 @@ xmlShellPrintXPathResult(xmlXPathObjectPtr list)
                              indx++) {
                             if (i > 0)
                                 fprintf(stderr, " -------\n");
-                            xmlShellPrintNode(list->nodesetval->
-                                              nodeTab[indx]);
+                            xmlShellPrintNodeCtxt(ctxt,
+				    list->nodesetval->nodeTab[indx]);
                         }
                     } else {
                         xmlGenericError(xmlGenericErrorContext,
@@ -1429,6 +1439,18 @@ xmlShellPrintXPathResult(xmlXPathObjectPtr list)
     }
 }
 
+/**
+ * xmlShellPrintXPathResult:
+ * @list: a valid result generated by an xpath evaluation
+ *
+ * Prints result to the output FILE
+ */
+void
+xmlShellPrintXPathResult(xmlXPathObjectPtr list)
+{
+    xmlShellPrintXPathResultCtxt(NULL, list);
+}
+
 /**
  * xmlShellList:
  * @ctxt:  the shell context
@@ -1442,29 +1464,31 @@ xmlShellPrintXPathResult(xmlXPathObjectPtr list)
  * Returns 0
  */
 int
-xmlShellList(xmlShellCtxtPtr ctxt ATTRIBUTE_UNUSED,
+xmlShellList(xmlShellCtxtPtr ctxt,
              char *arg ATTRIBUTE_UNUSED, xmlNodePtr node,
              xmlNodePtr node2 ATTRIBUTE_UNUSED)
 {
     xmlNodePtr cur;
+    if (!ctxt)
+        return (0);
     if (node == NULL) {
-	fprintf(stdout, "NULL\n");
+	fprintf(ctxt->output, "NULL\n");
 	return (0);
     }
     if ((node->type == XML_DOCUMENT_NODE) ||
         (node->type == XML_HTML_DOCUMENT_NODE)) {
         cur = ((xmlDocPtr) node)->children;
     } else if (node->type == XML_NAMESPACE_DECL) {
-        xmlLsOneNode(stdout, node);
+        xmlLsOneNode(ctxt->output, node);
         return (0);
     } else if (node->children != NULL) {
         cur = node->children;
     } else {
-        xmlLsOneNode(stdout, node);
+        xmlLsOneNode(ctxt->output, node);
         return (0);
     }
     while (cur != NULL) {
-        xmlLsOneNode(stdout, cur);
+        xmlLsOneNode(ctxt->output, cur);
         cur = cur->next;
     }
     return (0);
@@ -1483,22 +1507,24 @@ xmlShellList(xmlShellCtxtPtr ctxt ATTRIBUTE_UNUSED,
  * Returns 0
  */
 int
-xmlShellBase(xmlShellCtxtPtr ctxt ATTRIBUTE_UNUSED,
+xmlShellBase(xmlShellCtxtPtr ctxt,
              char *arg ATTRIBUTE_UNUSED, xmlNodePtr node,
              xmlNodePtr node2 ATTRIBUTE_UNUSED)
 {
     xmlChar *base;
+    if (!ctxt)
+        return 0;
     if (node == NULL) {
-	fprintf(stdout, "NULL\n");
+	fprintf(ctxt->output, "NULL\n");
 	return (0);
     }    
 
     base = xmlNodeGetBase(node->doc, node);
 
     if (base == NULL) {
-        fprintf(stdout, " No base found !!!\n");
+        fprintf(ctxt->output, " No base found !!!\n");
     } else {
-        fprintf(stdout, "%s\n", base);
+        fprintf(ctxt->output, "%s\n", base);
         xmlFree(base);
     }
     return (0);
@@ -1525,6 +1551,84 @@ xmlShellSetBase(xmlShellCtxtPtr ctxt ATTRIBUTE_UNUSED,
     return (0);
 }
 
+/**
+ * xmlShellGrep:
+ * @ctxt:  the shell context
+ * @arg:  the string or regular expression to find
+ * @node:  a node
+ * @node2:  unused
+ *
+ * Implements the XML shell function "grep"
+ * dumps informations about the node (namespace, attributes, content).
+ *
+ * Returns 0
+ */
+static int
+xmlShellGrep(xmlShellCtxtPtr ctxt ATTRIBUTE_UNUSED,
+            char *arg, xmlNodePtr node, xmlNodePtr node2 ATTRIBUTE_UNUSED)
+{
+    if (!ctxt)
+        return (0);
+    if (node == NULL)
+	return (0);
+    if (arg == NULL)
+	return (0);
+#ifdef LIBXML_REGEXP_ENABLED
+    if ((xmlStrchr((xmlChar *) arg, '?')) ||
+	(xmlStrchr((xmlChar *) arg, '*')) ||
+	(xmlStrchr((xmlChar *) arg, '.')) ||
+	(xmlStrchr((xmlChar *) arg, '['))) {
+    }
+#endif
+    while (node != NULL) {
+        if (node->type == XML_COMMENT_NODE) {
+	    if (xmlStrstr(node->content, (xmlChar *) arg)) {
+
+		fprintf(ctxt->output, "%s : ", xmlGetNodePath(node));
+                xmlShellList(ctxt, NULL, node, NULL);
+	    }
+        } else if (node->type == XML_TEXT_NODE) {
+	    if (xmlStrstr(node->content, (xmlChar *) arg)) {
+
+		fprintf(ctxt->output, "%s : ", xmlGetNodePath(node->parent));
+                xmlShellList(ctxt, NULL, node->parent, NULL);
+	    }
+        }
+
+        /*
+         * Browse the full subtree, deep first
+         */
+
+        if ((node->type == XML_DOCUMENT_NODE) ||
+            (node->type == XML_HTML_DOCUMENT_NODE)) {
+            node = ((xmlDocPtr) node)->children;
+        } else if ((node->children != NULL)
+                   && (node->type != XML_ENTITY_REF_NODE)) {
+            /* deep first */
+            node = node->children;
+        } else if (node->next != NULL) {
+            /* then siblings */
+            node = node->next;
+        } else {
+            /* go up to parents->next if needed */
+            while (node != NULL) {
+                if (node->parent != NULL) {
+                    node = node->parent;
+                }
+                if (node->next != NULL) {
+                    node = node->next;
+                    break;
+                }
+                if (node->parent == NULL) {
+                    node = NULL;
+                    break;
+                }
+            }
+	}
+    }
+    return (0);
+}
+
 /**
  * xmlShellDir:
  * @ctxt:  the shell context
@@ -1542,17 +1646,19 @@ xmlShellDir(xmlShellCtxtPtr ctxt ATTRIBUTE_UNUSED,
             char *arg ATTRIBUTE_UNUSED, xmlNodePtr node,
             xmlNodePtr node2 ATTRIBUTE_UNUSED)
 {
+    if (!ctxt)
+        return (0);
     if (node == NULL) {
-	fprintf(stdout, "NULL\n");
+	fprintf(ctxt->output, "NULL\n");
 	return (0);
     }    
     if ((node->type == XML_DOCUMENT_NODE) ||
         (node->type == XML_HTML_DOCUMENT_NODE)) {
-        xmlDebugDumpDocumentHead(stdout, (xmlDocPtr) node);
+        xmlDebugDumpDocumentHead(ctxt->output, (xmlDocPtr) node);
     } else if (node->type == XML_ATTRIBUTE_NODE) {
-        xmlDebugDumpAttr(stdout, (xmlAttrPtr) node, 0);
+        xmlDebugDumpAttr(ctxt->output, (xmlAttrPtr) node, 0);
     } else {
-        xmlDebugDumpOneNode(stdout, node, 0);
+        xmlDebugDumpOneNode(ctxt->output, node, 0);
     }
     return (0);
 }
@@ -1573,29 +1679,31 @@ int
 xmlShellCat(xmlShellCtxtPtr ctxt, char *arg ATTRIBUTE_UNUSED,
             xmlNodePtr node, xmlNodePtr node2 ATTRIBUTE_UNUSED)
 {
+    if (!ctxt)
+        return (0);
     if (node == NULL) {
-	fprintf(stdout, "NULL\n");
+	fprintf(ctxt->output, "NULL\n");
 	return (0);
     }    
     if (ctxt->doc->type == XML_HTML_DOCUMENT_NODE) {
 #ifdef LIBXML_HTML_ENABLED
         if (node->type == XML_HTML_DOCUMENT_NODE)
-            htmlDocDump(stdout, (htmlDocPtr) node);
+            htmlDocDump(ctxt->output, (htmlDocPtr) node);
         else
-            htmlNodeDumpFile(stdout, ctxt->doc, node);
+            htmlNodeDumpFile(ctxt->output, ctxt->doc, node);
 #else
         if (node->type == XML_DOCUMENT_NODE)
-            xmlDocDump(stdout, (xmlDocPtr) node);
+            xmlDocDump(ctxt->output, (xmlDocPtr) node);
         else
-            xmlElemDump(stdout, ctxt->doc, node);
+            xmlElemDump(ctxt->output, ctxt->doc, node);
 #endif /* LIBXML_HTML_ENABLED */
     } else {
         if (node->type == XML_DOCUMENT_NODE)
-            xmlDocDump(stdout, (xmlDocPtr) node);
+            xmlDocDump(ctxt->output, (xmlDocPtr) node);
         else
-            xmlElemDump(stdout, ctxt->doc, node);
+            xmlElemDump(ctxt->output, ctxt->doc, node);
     }
-    fprintf(stdout, "\n");
+    fprintf(ctxt->output, "\n");
     return (0);
 }
 
@@ -1626,7 +1734,7 @@ xmlShellLoad(xmlShellCtxtPtr ctxt, char *filename,
 #ifdef LIBXML_HTML_ENABLED
         doc = htmlParseFile(filename, NULL);
 #else
-        fprintf(stdout, "HTML support not compiled in\n");
+        fprintf(ctxt->output, "HTML support not compiled in\n");
         doc = NULL;
 #endif /* LIBXML_HTML_ENABLED */
     } else {
@@ -1833,24 +1941,27 @@ xmlShellValidate(xmlShellCtxtPtr ctxt, char *dtd,
  * Returns 0 or -1 in case of error
  */
 int
-xmlShellDu(xmlShellCtxtPtr ctxt ATTRIBUTE_UNUSED,
+xmlShellDu(xmlShellCtxtPtr ctxt,
            char *arg ATTRIBUTE_UNUSED, xmlNodePtr tree,
            xmlNodePtr node2 ATTRIBUTE_UNUSED)
 {
     xmlNodePtr node;
     int indent = 0, i;
 
+    if (!ctxt)
+	return (-1);
+
     if (tree == NULL)
         return (-1);
     node = tree;
     while (node != NULL) {
         if ((node->type == XML_DOCUMENT_NODE) ||
             (node->type == XML_HTML_DOCUMENT_NODE)) {
-            fprintf(stdout, "/\n");
+            fprintf(ctxt->output, "/\n");
         } else if (node->type == XML_ELEMENT_NODE) {
             for (i = 0; i < indent; i++)
-                fprintf(stdout, "  ");
-            fprintf(stdout, "%s\n", node->name);
+                fprintf(ctxt->output, "  ");
+            fprintf(ctxt->output, "%s\n", node->name);
         } else {
         }
 
@@ -1945,7 +2056,7 @@ xmlShellPwd(xmlShellCtxtPtr ctxt ATTRIBUTE_UNUSED, char *buffer,
  * @doc:  the initial document
  * @filename:  the output buffer
  * @input:  the line reading function
- * @output:  the output FILE*
+ * @output:  the output FILE*, defaults to stdout if NULL
  *
  * Implements the XML shell 
  * This allow to load, validate, view, modify and save a document
@@ -1971,7 +2082,7 @@ xmlShell(xmlDocPtr doc, char *filename, xmlShellReadlineFunc input,
     if (input == NULL)
         return;
     if (output == NULL)
-        return;
+        output = stdout;
     ctxt = (xmlShellCtxtPtr) xmlMalloc(sizeof(xmlShellCtxt));
     if (ctxt == NULL)
         return;
@@ -1991,11 +2102,11 @@ xmlShell(xmlDocPtr doc, char *filename, xmlShellReadlineFunc input,
 #endif /* LIBXML_XPATH_ENABLED */
     while (1) {
         if (ctxt->node == (xmlNodePtr) ctxt->doc)
-            sprintf(prompt, "%s > ", "/");
+            snprintf(prompt, sizeof(prompt), "%s > ", "/");
         else if (ctxt->node->name)
             snprintf(prompt, sizeof(prompt), "%s > ", ctxt->node->name);
         else
-            sprintf(prompt, "? > ");
+            snprintf(prompt, sizeof(prompt), "? > ");
         prompt[sizeof(prompt) - 1] = 0;
 
         /*
@@ -2049,26 +2160,27 @@ xmlShell(xmlDocPtr doc, char *filename, xmlShellReadlineFunc input,
         if (!strcmp(command, "bye"))
             break;
 		if (!strcmp(command, "help")) {
-		  fprintf(stdout, "\tbase         display XML base of the node\n");
-		  fprintf(stdout, "\tsetbase URI  change the XML base of the node\n");
-		  fprintf(stdout, "\tbye          leave shell\n");
-		  fprintf(stdout, "\tcat [node]   display node or current node\n");
-		  fprintf(stdout, "\tcd [path]    change directory to path or to root\n");
-		  fprintf(stdout, "\tdir [path]   dumps informations about the node (namespace, attributes, content)\n");
-		  fprintf(stdout, "\tdu [path]    show the structure of the subtree under path or the current node\n");
-		  fprintf(stdout, "\texit         leave shell\n");
-		  fprintf(stdout, "\thelp         display this help\n");
-		  fprintf(stdout, "\tfree         display memory usage\n");
-		  fprintf(stdout, "\tload [name]  load a new document with name\n");
-		  fprintf(stdout, "\tls [path]    list contents of path or the current directory\n");
+		  fprintf(ctxt->output, "\tbase         display XML base of the node\n");
+		  fprintf(ctxt->output, "\tsetbase URI  change the XML base of the node\n");
+		  fprintf(ctxt->output, "\tbye          leave shell\n");
+		  fprintf(ctxt->output, "\tcat [node]   display node or current node\n");
+		  fprintf(ctxt->output, "\tcd [path]    change directory to path or to root\n");
+		  fprintf(ctxt->output, "\tdir [path]   dumps informations about the node (namespace, attributes, content)\n");
+		  fprintf(ctxt->output, "\tdu [path]    show the structure of the subtree under path or the current node\n");
+		  fprintf(ctxt->output, "\texit         leave shell\n");
+		  fprintf(ctxt->output, "\thelp         display this help\n");
+		  fprintf(ctxt->output, "\tfree         display memory usage\n");
+		  fprintf(ctxt->output, "\tload [name]  load a new document with name\n");
+		  fprintf(ctxt->output, "\tls [path]    list contents of path or the current directory\n");
 #ifdef LIBXML_XPATH_ENABLED
-		  fprintf(stdout, "\txpath expr   evaluate the XPath expression in that context and print the result\n");
+		  fprintf(ctxt->output, "\txpath expr   evaluate the XPath expression in that context and print the result\n");
 #endif /* LIBXML_XPATH_ENABLED */
-		  fprintf(stdout, "\tpwd          display current working directory\n");
-		  fprintf(stdout, "\tquit         leave shell\n");
-		  fprintf(stdout, "\tsave [name]  save this document to name or the original name\n");
-		  fprintf(stdout, "\tvalidate     check the document for errors\n");
-		  fprintf(stdout, "\twrite [name] write the current node to the filename\n");
+		  fprintf(ctxt->output, "\tpwd          display current working directory\n");
+		  fprintf(ctxt->output, "\tquit         leave shell\n");
+		  fprintf(ctxt->output, "\tsave [name]  save this document to name or the original name\n");
+		  fprintf(ctxt->output, "\tvalidate     check the document for errors\n");
+		  fprintf(ctxt->output, "\twrite [name] write the current node to the filename\n");
+		  fprintf(ctxt->output, "\tgrep string  search for a string in the subtree\n");
         } else if (!strcmp(command, "validate")) {
             xmlShellValidate(ctxt, arg, NULL, NULL);
         } else if (!strcmp(command, "load")) {
@@ -2077,20 +2189,22 @@ xmlShell(xmlDocPtr doc, char *filename, xmlShellReadlineFunc input,
             xmlShellSave(ctxt, arg, NULL, NULL);
         } else if (!strcmp(command, "write")) {
             xmlShellWrite(ctxt, arg, NULL, NULL);
+        } else if (!strcmp(command, "grep")) {
+            xmlShellGrep(ctxt, arg, ctxt->node, NULL);
         } else if (!strcmp(command, "free")) {
             if (arg[0] == 0) {
-                xmlMemShow(stdout, 0);
+                xmlMemShow(ctxt->output, 0);
             } else {
                 int len = 0;
 
                 sscanf(arg, "%d", &len);
-                xmlMemShow(stdout, len);
+                xmlMemShow(ctxt->output, len);
             }
         } else if (!strcmp(command, "pwd")) {
             char dir[500];
 
             if (!xmlShellPwd(ctxt, dir, ctxt->node, NULL))
-                fprintf(stdout, "%s\n", dir);
+                fprintf(ctxt->output, "%s\n", dir);
         } else if (!strcmp(command, "du")) {
             xmlShellDu(ctxt, NULL, ctxt->node, NULL);
         } else if (!strcmp(command, "base")) {
@@ -2103,7 +2217,7 @@ xmlShell(xmlDocPtr doc, char *filename, xmlShellReadlineFunc input,
 	    } else {
                 ctxt->pctxt->node = ctxt->node;
                 list = xmlXPathEval((xmlChar *) arg, ctxt->pctxt);
-		xmlXPathDebugDumpObject(stdout, list, 0);
+		xmlXPathDebugDumpObject(ctxt->output, list, 0);
 		xmlXPathFreeObject(list);
 	    }
 #endif /* LIBXML_XPATH_ENABLED */
@@ -2294,7 +2408,7 @@ xmlShell(xmlDocPtr doc, char *filename, xmlShellReadlineFunc input,
                                      indx < list->nodesetval->nodeNr;
                                      indx++) {
                                     if (i > 0)
-                                        fprintf(stdout, " -------\n");
+                                        fprintf(ctxt->output, " -------\n");
                                     xmlShellCat(ctxt, NULL,
                                                 list->nodesetval->
                                                 nodeTab[indx], NULL);
